import { TimeInterval } from '../types/types';
const dayInMs = 1000 * 60 * 60 * 24;
export class ChartDataService {
    static dataFormatter(type, dataPoint) {
        switch (type) {
            case 'date':
                return new Date(dataPoint);
            case 'number':
            case 'percent':
            case 'string':
            case 'currency':
            default:
                return dataPoint;
        }
    }
    static formatData(axisConfigs, data) {
        return data.map(d => {
            const x = ChartDataService.dataFormatter(axisConfigs.x.labelType, d.x);
            d.x = x || d.name;
            d.y = ChartDataService.dataFormatter(axisConfigs.y.labelType, d.y);
            return d;
        });
    }
    static xFormatter(period, axisConfig, value) {
        return ChartDataService.axisLabelFormatter(value, axisConfig, period);
    }
    static yFormatter(axisConfig, value) {
        return ChartDataService.axisLabelFormatter(value, axisConfig);
    }
    static axisLabelFormatter(value, axisConfig, period) {
        let currency = ChartDataService.DEFAULT_CURRENCY;
        let locale = ChartDataService.DEFAULT_LOCALE;
        const type = axisConfig.labelType;
        if (axisConfig.formattingOptions) {
            locale = axisConfig.formattingOptions.locale || ChartDataService.DEFAULT_LOCALE;
            currency = axisConfig.formattingOptions.currency || ChartDataService.DEFAULT_CURRENCY;
        }
        switch (type) {
            case 'currency':
                return new Intl.NumberFormat(locale, { style: 'currency', currency }).format(value);
            case 'percent':
                return `${value.toString()}%`;
            case 'number':
                return new Intl.NumberFormat(locale).format(value);
            case 'date':
                return ChartDataService.dateFormatter(value, locale, period);
            case 'string':
            default:
                return value.toString();
        }
    }
    static dateFormatter(value, locale, period) {
        const valueValidDate = !!Date.parse(value);
        if (!valueValidDate) {
            return value.toString();
        }
        // Handle date so bleed doesn't happen to next/previous month
        const date = new Date(value);
        const UTCyear = date.getUTCFullYear();
        const UTCmonth = date.getUTCMonth();
        const UTCday = date.getUTCDate();
        const formatParts = new Intl.DateTimeFormat(locale, { day: '2-digit', month: 'short', year: 'numeric' }).formatToParts(new Date(UTCyear, UTCmonth, UTCday));
        const day = formatParts.find(p => p.type === 'day').value.toString();
        const month = formatParts.find(p => p.type === 'month').value.toString();
        const year = formatParts.find(p => p.type === 'year').value.toString();
        switch (period) {
            case TimeInterval.Yearly:
                return year;
            case TimeInterval.Weekly:
            case TimeInterval.Daily:
                return `${day} ${month}`;
            default:
                return `${month} ${year}`;
        }
    }
    static calculateInterval(data) {
        const dayPeriod = 1000 * 60 * 60 * 24;
        let interval = TimeInterval.Monthly;
        if (data.length < 2) {
            return interval;
        }
        const date1 = new Date(data[0].x);
        const date2 = new Date(data[1].x);
        if (isNaN(date1.valueOf()) || isNaN(date2.valueOf())) {
            return interval;
        }
        const timeElapsed = date2.valueOf() - date1.valueOf();
        // bucket these loosely so that a few hours doesn't break things if the dates aren't in lockstep
        if (timeElapsed < dayPeriod * 2) {
            interval = TimeInterval.Daily;
        }
        else if (timeElapsed < dayPeriod * 27) {
            interval = TimeInterval.Weekly;
        }
        else if (timeElapsed > dayPeriod * 364) {
            interval = TimeInterval.Yearly;
        }
        return interval;
    }
    static determineDateRange(data) {
        const domainPadding = this.domainPadding(data);
        const minDate = data.reduce((r, p) => {
            if (r === null) {
                return +p.x;
            }
            return Math.min(r, +p.x);
        }, null);
        const maxDate = data.reduce((r, p) => {
            if (r === null) {
                return +p.x;
            }
            return Math.max(r, +p.x);
        }, null);
        return [new Date(+minDate - domainPadding), new Date(+maxDate + domainPadding)];
    }
    static domainPadding(data) {
        const interval = ChartDataService.calculateInterval(data);
        switch (interval) {
            case TimeInterval.Daily:
                return this.halfDayMs;
            case TimeInterval.Weekly:
                return this.halfWeekMs;
            case TimeInterval.Monthly:
                return this.halfMonthMs;
            case TimeInterval.Yearly:
                return this.halfYearMs;
            default:
                return this.halfMonthMs;
        }
    }
}
ChartDataService.DEFAULT_CURRENCY = 'USD';
ChartDataService.DEFAULT_LOCALE = 'en-US';
ChartDataService.halfDayMs = dayInMs / 2;
ChartDataService.halfWeekMs = dayInMs * (7 / 2);
ChartDataService.halfMonthMs = dayInMs * (30.42 / 2);
ChartDataService.halfYearMs = dayInMs * (365 / 2);
